import numpy as np
import sklearn.model_selection as model_selection
import matplotlib.pyplot as plt
import sklearn.metrics as metrics
import cv2

# Hay dos formas de leer un achivo
# CSV con numpy

# np.loadtxt
# np.genfromtxt 

D = np.genfromtxt( "iris.csv", delimiter=',', skip_header=1 )
# print( D.shape )
# print( D )

# Los vectores de características
Xtodo = D[:,0:4] # 0 1 2 3 4
ytodo = D[:,4] 

# Un arreglo de True y False
indices = ytodo != 2

X = Xtodo[ indices, 0:2 ].astype( np.float32 )
y = ytodo[ indices ].astype( np.float32 )
print( X.shape )
print( y.shape )

# Aquí ya tenemos nuestros datos
# Los separamos en datos de entrenamiento y de prueba:
Xentrena, Xprueba, yentrena, yprueba = model_selection.train_test_split( X, y, test_size = 0.2, random_state = 6 )

regLog = cv2.ml.LogisticRegression_create( )
regLog.setTrainMethod( cv2.ml.LogisticRegression_MINI_BATCH )
regLog.setIterations( 100 )
regLog.setMiniBatchSize( 1 )
print( "Reg:", regLog.getRegularization() )
# Por defecto está ya seleccionada la regularización L2

regLog.setLearningRate( 0.0018 )
print( "LR:", regLog.getLearningRate() )

regLog.train( Xentrena, cv2.ml.ROW_SAMPLE, yentrena )

modelo = regLog.get_learnt_thetas()
print( modelo )

ret, ypredichas = regLog.predict( Xentrena )
exactitud = metrics.accuracy_score( yentrena, ypredichas )
print( "Entrenamiento:", exactitud )

ret, ypredichas = regLog.predict( Xprueba )
e = metrics.accuracy_score( yprueba, ypredichas )
print( "Prueba:", e )

# Dibujamos la línea de separación de las clases
a0 = modelo[0,0]
a1 = modelo[0,1]
a2 = modelo[0,2]

vx = np.linspace( 4, 7, 10 ) 
vy = (- a0 - a1*vx)/a2

fig, [ax1, ax2] = plt.subplots( 1, 2 )

ax1.set_title("BD Iris, datos de entrenamiento")
ax1.set_xlabel( "Largo (cm)" )
ax1.set_ylabel( "Ancho (cm)" )
ax1.scatter( Xentrena[:,0], Xentrena[:,1], c=yentrena )
ax1.plot( vx, vy )

ax2.set_title("BD Iris, datos de evaluación")
ax2.scatter( Xprueba[:,0], Xprueba[:,1], c=yprueba )
ax2.plot( vx, vy )

plt.show( )


