import numpy as np
import sklearn.model_selection as model_selection
import matplotlib.pyplot as plt
import sklearn.metrics as metrics
import cv2

# Hay dos formas de leer un achivo
# CSV con numpy

# np.loadtxt
# np.genfromtxt 

D = np.genfromtxt( "iris.csv", delimiter=',', skip_header=1 )
# print( D.shape )
# print( D )

# Los vectores de características
Xtodo = D[:,0:4] # 0 1 2 3 4
ytodo = D[:,4] 

# Un arreglo de True y False
indices = ytodo != 2

X = Xtodo[ indices, 0:2 ].astype( np.float32 )
y = ytodo[ indices ].astype( np.float32 )
print( X.shape )
print( y.shape )

# Aquí ya tenemos nuestros datos
# Los separamos en datos de entrenamiento y de prueba:
Xentrena, Xprueba, yentrena, yprueba = model_selection.train_test_split( X, y, test_size = 0.2, random_state = 6 )

kNN = cv2.ml.KNearest_create( )

kNN.train( Xentrena, cv2.ml.ROW_SAMPLE, yentrena )

ret, ypredichas, neighborResponses, dist = kNN.findNearest( Xentrena, 3 )

exactitud = metrics.accuracy_score( yentrena, ypredichas )
print( "Entrenamiento:", exactitud )

ret, ypredichas, neighborResponses, dist =  kNN.findNearest( Xprueba, 3 )
e = metrics.accuracy_score( yprueba, ypredichas )
print( "Prueba:", e )

# print( ypredichas )
# print( yprueba.shape )
# print( ypredichas.shape )

fig, [ax1, ax2, ax3] = plt.subplots( 1, 3 )

ax1.set_title("Entrenamiento")
ax1.set_xlabel( "Largo (cm)" )
ax1.set_ylabel( "Ancho (cm)" )
ax1.scatter( Xentrena[:,0], Xentrena[:,1], c=yentrena )
xmin, xmax = ax1.get_xlim( )
ymin, ymax = ax1.get_ylim( )

ax2.set_title("Evaluación")
ax2.set_xlim( xmin, xmax )
ax2.set_ylim( ymin, ymax )
ax2.scatter( Xprueba[:,0], Xprueba[:,1], c=yprueba )

ax3.set_title("Predicción")
ax3.set_xlim( xmin, xmax )
ax3.set_ylim( ymin, ymax )
# ax3.scatter( Xprueba[:,0], Xprueba[:,1], c=ypredichas.reshape( (20,))  )
ax3.scatter( Xprueba[:,0], Xprueba[:,1], c=ypredichas.reshape( (-1,))  )

plt.show( )

